@@ -31,7 +31,7 @@ def backup_dotfiles(backup_dest_path, dry_run=False, home_path=os.path.expanduse
 	dot_path_pairs = []
 	for dotfile_path_from_config, options in config.items():
 		# Evaluate condition, if specified. Skip if the command doesn't return true.
-		condition_success = evaluate_condition(condition=options["backup_condition"],
+		condition_success = evaluate_condition(condition=options.get("backup_condition", ""),
 											   backup_or_reinstall="backup",
 											   dotfile_path=dotfile_path_from_config)
 		if not condition_success:
@@ -85,7 +85,7 @@ def backup_dotfiles(backup_dest_path, dry_run=False, home_path=os.path.expanduse
 
 		print_blue_bold("Backing up dotfiles...")
 		for x in dotfiles_mp_in:
-			p = mp.Process(target=copyfile, args=(x[0], x[1],))
+			p = mp.Process(target=copyfile_with_exception_handler, args=(x[0], x[1],))
 			p.start()
 			p.join()
 
@@ -138,16 +138,23 @@ def backup_packages(backup_path, dry_run: bool = False, skip=False):
 	if not dry_run:
 		overwrite_dir_prompt_if_needed(backup_path, skip)
 
-	for mgr in ["brew", "brew cask", "gem"]:
-		# deal with package managers that have spaces in them.
-		print_pkg_mgr_backup(mgr)
-		command = f"{mgr} list"
-		dest = f"{backup_path}/{mgr.replace(' ', '-')}_list.txt"
-		run_cmd_if_no_dry_run(command, dest, dry_run)
+	# brew
+	print_pkg_mgr_backup("brew")
+	command = f"brew bundle dump --file {backup_path}/brew_list.txt"
+	dest = f"{backup_path}/brew_list.txt"
+	if not dry_run:
+		if not run_cmd(command):
+			print_yellow("brew package manager not found.")
+
+	# ruby
+	print_pkg_mgr_backup("gem")
+	command = r"gem list | tail -n+1 | sed -E 's/\((default: )?(.*)\)/--version \2/'"
+	dest = f"{backup_path}/gem_list.txt"
+	run_cmd_if_no_dry_run(command, dest, dry_run)
 
 	# cargo
 	print_pkg_mgr_backup("cargo")
-	command = "ls {}".format(home_prefix(".cargo/bin/"))
+	command = r"cargo install --list | grep '^\w.*:$' | sed -E 's/ v(.*):$/ --version \1/'"
 	dest = f"{backup_path}/cargo_list.txt"
 	run_cmd_if_no_dry_run(command, dest, dry_run)
 
@@ -168,7 +175,7 @@ def backup_packages(backup_path, dry_run: bool = False, skip=False):
 	command = "npm ls --global --parseable=true --depth=0"
 	temp_file_path = f"{backup_path}/npm_temp_list.txt"
 	# If command is successful, go to the next parsing step.
-	npm_backup_cmd_success = run_cmd_if_no_dry_run(command, dest, dry_run) == 0
+	npm_backup_cmd_success = run_cmd_if_no_dry_run(command, temp_file_path, dry_run) == 0
 	if npm_backup_cmd_success:
 		npm_dest_file = f"{backup_path}/npm_list.txt"
 		# Parse npm output
